MODULE the_work
!
CONTAINS
SUBROUTINE initialize (generator, power, ngen)
!
IMPLICIT NONE
REAL   , DIMENSION(4,4,3), INTENT(OUT) :: generator ! Define with INTENT(OUT)
INTEGER, DIMENSION(3)    , INTENT(OUT) :: power     ! to prevent errors in which these 
INTEGER                  , INTENT(OUT) :: ngen      ! variables are used, not written
!
ngen             = 3       ! C2/c has three generators
generator        = 0       ! Initialize all elements to zero
generator(4,4,:) = 1.      ! Initialize all (4,4) elements to 1.
power            = 1       ! Initialize all powers to 1
!
!  (2)  2  0,y,1/4           ==> -x, y, -z+1/2
generator(1,1,1) = -1.     ! All other elements are zero
generator(2,2,1) =  1.     ! values derived from I.T.
generator(3,3,1) = -1.
generator(3,4,1) =  0.5
power    (    1) =  1
!
!  (3) -1  0,0,0           ==> -x, -y, -z
generator(1,1,2) = -1.
generator(2,2,2) = -1.
generator(3,3,2) = -1.
generator(3,4,2) =  0.
power    (    2) =  1
!
!       t (1/2, 1/2, 0)    ==> x+1/2, y+1/2, z
generator(1,1,3) =  1.
generator(2,2,3) =  1.
generator(3,3,3) =  1.
generator(1,4,3) =  0.5
generator(2,4,3) =  0.5
power    (    3) =  1
!
END SUBROUTINE initialize
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
SUBROUTINE create(symmetry, nsym, generator, power, ngen)
!
IMPLICIT NONE
! declare all dymmy arguments
REAL   , DIMENSION(4,4,8), INTENT(OUT) :: symmetry   ! output variables
INTEGER                  , INTENT(OUT) :: nsym
REAL   , DIMENSION(4,4,3), INTENT(IN ) :: generator  ! these are input
INTEGER, DIMENSION(3)    , INTENT(IN ) :: power      ! prevent accidental
INTEGER                  , INTENT(IN ) :: ngen       ! overwrite
! local variables follow
INTEGER                                :: igen, ipow, isym, j  ! dummy loop indices
REAL, DIMENSION(4,4)                   :: temp      = 0.0
REAL, DIMENSION(4,4),PARAMETER         :: unity =  & ! a PARAMETER is fixed
    RESHAPE((/1.,0.,0.,0., 0.,1.,0.,0., 0.,0.,1.,0., 0.,0.,0.,1./), SHAPE(unity))
!
symmetry(:,:,1) = unity                ! Set first symmetry operation to a unit matrix
nsym            = 1                    ! Now we have one symmetry operation
!
j = nsym                               ! j is used to count up the number of Sym.Op.
!
loop_gen: DO igen=1,ngen               ! Loop over all generators here 3
   temp = unity                        ! Set temp to a unit matrix
   loop_power: DO ipow=1,power(igen)   ! Loop over al power, here 1
      temp = MATMUL(generator(:,:,igen), temp)  ! calculate actual generator
      loop_sym: DO isym = 1, nsym      ! Loop over all previous Sym.Op.
         j = j + 1                     ! increment total number of Sym.Op.
         symmetry(:,:,j) = MATMUL(temp, symmetry(:,:,isym)) ! calc new Sym.Op.
      END DO loop_sym
      nsym = j                         ! Update the number of Sym.Op.
   END DO loop_power
END DO loop_gen
!
END SUBROUTINE create
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
SUBROUTINE printsym(symmetry, nsym)  ! Print all symmetry operations
!
IMPLICIT NONE
!
INTEGER                     , INTENT(IN) :: nsym
REAL   , DIMENSION(4,4,nsym), INTENT(IN) :: symmetry
!
INTEGER                :: isym, i,j
!
WRITE(*,1000)
loop_sym: DO isym = 1, nsym
   WRITE(*,2000) isym, ((symmetry(i,j,isym), j=1,4), i=1,3)
END DO loop_sym
!
1000 FORMAT('Symmetry Matrices in C2/c ',/)
2000 FORMAT(' Matrix No :', I4,2x,          &
             ( 3(F3.0,', '),'  ',F5.2 ,/)    &
        2(18x, 3(F3.0,', '),'  ',F5.2 ,/)    &
           )
END SUBROUTINE printsym
END MODULE the_work
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
PROGRAM generate_C2_c
!
USE the_work     ! This contains all subroutines, helps to reduce errors
IMPLICIT NONE
!
INTEGER                   :: ngen      = 1    ! Number of generators, initialize to one
INTEGER                   :: nsym      = 1    ! Number of symmetry op., initialize to one
REAL   , DIMENSION(4,4,8) :: symmetry  = 0.0  ! Array to hold all symmetry operations
REAL   , DIMENSION(4,4,3) :: generator = 0.0  ! Array to hold all generators
INTEGER, DIMENSION(3)     :: power     = 1    ! Array to hold generator powers
!
CALL initialize(generator, power, ngen)
!
CALL create(symmetry, nsym, generator, power, ngen)
!
CALL printsym(symmetry, nsym)
!
END PROGRAM generate_C2_c
