MODULE the_work
!
PRIVATE
PUBLIC initialize, input, create, printatom
!
CONTAINS
!
SUBROUTINE initialize (generator, power, ngen)
!
IMPLICIT NONE
REAL   , DIMENSION(4,4,3), INTENT(OUT) :: generator ! Define with INTENT(OUT)
INTEGER, DIMENSION(3)    , INTENT(OUT) :: power     ! to prevent errors in which these 
INTEGER                  , INTENT(OUT) :: ngen      ! variables are used, not written
!
ngen             = 3       ! C2/c has three generators
generator        = 0       ! Initialize all elements to zero
generator(4,4,:) = 1.      ! Initialize all (4,4) elements to 1.
power            = 1       ! Initialize all powers to 1
!
!  (2)  2  0,y,1/4           ==> -x, y, -z+1/2
generator(1,1,1) = -1.     ! All other elements are zero
generator(2,2,1) =  1.     ! values derived from I.T.
generator(3,3,1) = -1.
generator(3,4,1) =  0.5
power    (    1) =  1
!
!  (3) -1  0,0,0           ==> -x, -y, -z
generator(1,1,2) = -1.
generator(2,2,2) = -1.
generator(3,3,2) = -1.
generator(3,4,2) =  0.
power    (    2) =  1
!
!       t (1/2, 1/2, 0)    ==> x+1/2, y+1/2, z
generator(1,1,3) =  1.
generator(2,2,3) =  1.
generator(3,3,3) =  1.
generator(1,4,3) =  0.5
generator(2,4,3) =  0.5
power    (    3) =  1
!
END SUBROUTINE initialize
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
SUBROUTINE input( atom )
REAL   , DIMENSION(4,8), INTENT(OUT)   :: atom
!
WRITE(*,1000,ADVANCE='NO') 
READ (*,*) atom(1:3,1)
atom(4,1) = 1.0
!
1000 FORMAT(' Give atom coordinates ')
END SUBROUTINE input
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
LOGICAL FUNCTION equal(old, new)  ! Compare old and new atom positions for equality
!
IMPLICIT NONE
!
REAL   , DIMENSION(4,1), INTENT(IN) :: old
REAL   , DIMENSION(4,1), INTENT(IN) :: new
!
REAL, PARAMETER :: EPS = 1.E-6
INTEGER :: i
!
equal = .true.
DO i = 1,3
   equal = equal .AND. ABS( old(i,1) - new(i,1)) < EPS
END DO
!
END FUNCTION equal
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
SUBROUTINE create(atom, natom, generator, power, ngen)
!
IMPLICIT NONE
! declare all dymmy arguments
REAL   , DIMENSION(4,  8), INTENT(OUT) :: atom       ! output variables
INTEGER                  , INTENT(OUT) :: natom    
REAL   , DIMENSION(4,4,3), INTENT(IN ) :: generator  ! these are input
INTEGER, DIMENSION(3)    , INTENT(IN ) :: power      ! prevent accidental
INTEGER                  , INTENT(IN ) :: ngen       ! overwrite
! local variables follow
INTEGER                                :: igen, ipow, iatom, j, k     ! dummy loop indices
REAL, DIMENSION(4,4)                   :: temp      = 0.0
REAL, DIMENSION(4,4),PARAMETER         :: unity =  & ! a PARAMETER is fixed
    RESHAPE((/1.,0.,0.,0., 0.,1.,0.,0., 0.,0.,1.,0., 0.,0.,0.,1./), SHAPE(unity))
!
CALL firstcell(atom(:,1))              ! shift first atom into interval [0:1[
natom           = 1                    ! Now we have one atom
!
j = natom                              ! j is used to count up the number of atoms
!
loop_gen: DO igen=1,ngen               ! Loop over all generators here 3
   temp = unity                        ! Set temp to a unit matrix
   loop_power: DO ipow=1,power(igen)   ! Loop over all power, here 1
      temp = MATMUL(generator(:,:,igen), temp)  ! calculate actual generator
      loop_prev: DO iatom = 1, natom   ! Loop over all previous atoms 
         j = j + 1                     ! increment total number of atoms 
         atom    (:,  j) = MATMUL(temp, atom(:,iatom)) ! calc new atoms
         CALL firstcell(atom(:,j))     ! Shift into first unit cell
         loop_comp: DO k=1, natom      ! Compare to all previous atoms
            IF(equal(atom(:,k),atom(:,j))) THEN
               j = j - 1               ! This position exists already, decrement
               CYCLE loop_gen          ! Skip generator go to next
            ENDIF
         END DO loop_comp
      END DO loop_prev
      natom = j                        ! Update the number of atoms
   END DO loop_power
END DO loop_gen
!
END SUBROUTINE create
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
SUBROUTINE firstcell(atom)         ! Move atom into first unit cell 0 <= xyz < 1
!
! Shift atom coordinates into interval [0:1[
! x   =  x   - FLOOR(x)   ! FLOOR(x) ist largest Integer <= x
! 0.3 =  1.3 - FLOOR( 1.3) =  1.3 -   1
! 0.3 = -1.7 - FLOOR(-1.7) = -1.7 - (-2)
!
IMPLICIT NONE
!
REAL   , DIMENSION(4,1), INTENT(INOUT) :: atom
!
INTEGER :: i
!
DO i = 1,3
   atom(i,1) = atom(i,1) - FLOOR ( atom(i,1))
END DO
!
END SUBROUTINE firstcell
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
SUBROUTINE printatom(atom, natom)  ! Print all atom position
!
IMPLICIT NONE
!
INTEGER                    , INTENT(IN) :: natom
REAL   , DIMENSION(4,natom), INTENT(IN) :: atom
!
INTEGER                :: iatom, i,j
!
WRITE(*,1000)
loop_atom: DO iatom = 1, natom
   WRITE(*,2000) iatom, (atom(i,iatom), i=1,3)
END DO loop_atom
!
1000 FORMAT('Atoms in C2/c ',/)
2000 FORMAT(' Atom No :', I4,2x, 3(F8.3,:', '))
END SUBROUTINE printatom
!
END MODULE the_work
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
PROGRAM makeatom_C2_c
!
USE the_work     ! This contains all subroutines, helps to reduce errors
IMPLICIT NONE
!
INTEGER                   :: ngen      = 1    ! Number of generators, initialize to one
REAL   , DIMENSION(4,4,3) :: generator = 0.0  ! Array to hold all generators
INTEGER, DIMENSION(3)     :: power     = 1    ! Array to hold generator powers
REAL   , DIMENSION(4,8)   :: atom      = 0.   ! Array of all atom positions
INTEGER                   :: natom     = 0    ! Number of aroms
!
CALL initialize(generator, power, ngen)
!
CALL input(atom)
!
CALL create(atom, natom, generator, power, ngen)
!
CALL printatom(atom, natom)
!
END PROGRAM makeatom_C2_c
